Unit SF42R_p;
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc, UTSF12_p, SF35R_p;

function SF42R(F :Real; N1 :Integer; N2 :Integer;
                var IERR :Integer): Real;

implementation

function SF42R(F :Real; N1 :Integer; N2 :Integer;
                var IERR :Integer): Real;
var
I1,I2,L2,I2P,I :Integer;
F1,F2,DP,VP,CBR2,S,DPL,THETA,X3,F1F,F2P,X1,X2,CBR1,STS,A,B,XI,C :Real;
const
NMAX :Integer = 20000;
BIG :Real = 1.0E13;
NMAX1 :Integer = 20000;
SYS015 :Real = 0.63661977236758134307553505349006;
SYS059 :Real = 3.4E38;
SYS029 :Real = 0.70710678118654752440084436210485;
SYS037 :Real = 0.33333333333333333333333333333333;
label
_1,_2,_3,_4,_5,_6,_7,_8,_9,_10,_11,_12,_13,_14,_15,_16,_17,_18,_19,
_20,_21,_22,_23,_24,_25;
begin
{ Result -    SF42R  FORTRANe }
if ( N1 < 1 ) or ( N2 < 1 ) or ( N1+N2 > NMAX ) 
 then goto _20;
if ( F < 0.0 ) 
 then goto _21;
IERR := 0;
if ( F = 0.0 ) 
 then goto _23;
F1 := N1;
F2 := N2;
DP := 0.0;
VP := F1+F2-2.0;
F1F := F1*F;
F2P := F2+F1F;
X1 := F2/F2P;
X2 := 1.0-X1;
if ( X2 = 0.0 ) 
 then goto _23;
if ( (N1 div 2)*2-N1 = 0 ) and ( N1 <= NMAX1 ) 
 then goto _1;
if ( (N2 div 2)*2-N2 = 0 ) and ( N2 <= NMAX1 ) 
 then goto _6;
if ( N1+N2 <= NMAX1 ) 
 then goto _11;
F1 := 0.2222222/F1;
F2 := 0.2222222/F2;
CBR1 := SYS037*Ln(F);
CBR1 := Exp(CBR1);
CBR2 := CBR1*CBR1;
S := (CBR1*(1.0-F2)-1.0+F1)/Sqrt(F1+CBR2*F2);
Result := 0.5*SF35R(-SYS029*S);
goto _25;
_1:
if ( N1 = 2 ) 
 then goto _5;
I1 := N1-2;
I2 := 2;
while ( I2<=I1 ) do
 begin
  L2 := I2;
  I := N1-I2;
  VP := VP-2.0;
  DP := X2*VP/I*(1.0+DP);
  if ( DP > BIG ) 
   then goto _3;
_2:
  inc(I2,2);
 end;
goto _5;
_3:
if ( L2 >= I1 ) 
 then goto _5;
DPL := Ln(DP);
I2P := L2+2;
I2 := I2P;
while ( I2<=I1 ) do
 begin
  I := N1-I2;
  VP := VP-2.0;
  DPL := DPL+Ln(X2*VP/I);
_4:
  inc(I2,2);
 end;
X3 := 1.0-Exp(0.5*F2*Ln(X1)+DPL);
Result := X3;
goto _19;
_5:
X2 := Exp(0.5*F2*Ln(X1));
X3 := 1.0-X2*(1.0+DP);
Result := X3;
goto _19;
_6:
if ( N2 = 2 ) 
 then goto _10;
I1 := N2-2;
I2 := 2;
while ( I2<=I1 ) do
 begin
  L2 := I2;
  I := N2-I2;
  VP := VP-2.0;
  DP := X1*VP/I*(1.0+DP);
  if ( DP > BIG ) 
   then goto _8;
_7:
  inc(I2,2);
 end;
goto _10;
_8:
if ( L2 >= I1 ) 
 then goto _10;
DPL := Ln(DP);
I2P := L2+2;
I2 := I2P;
while ( I2<=I1 ) do
 begin
  I := N2-I2;
  VP := VP-2.0;
  DPL := DPL+Ln(X1*VP/I);
_9:
  inc(I2,2);
 end;
X3 := Exp(0.5*F1*Ln(X2)+DPL);
Result := X3;
goto _19;
_10:
X2 := Exp(0.5*F1*Ln(X2));
X3 := X2*(1.0+DP);
Result := X3;
goto _19;
_11:
DP := Sqrt(F1F/F2);
THETA := ArcTan(DP);
STS := F1F/F2P;
A := 0.0;
B := 0.0;
if ( N2 = 1 ) 
 then goto _14;
if ( N2 = 3 ) 
 then goto _13;
I1 := N2-3;
I2 := 2;
while ( I2<=I1 ) do
 begin
  XI := F2-I2;
  A := X1*(XI-1.0)/XI*(1.0+A);
_12:
  inc(I2,2);
 end;
_13:
A := X1*DP*(1.0+A);
_14:
A := A+THETA;
if ( N1 = 1 ) 
 then goto _18;
if ( N1 = 3 ) 
 then goto _16;
I1 := N1-3;
I2 := 2;
while ( I2<=I1 ) do
 begin
  XI := F1-I2;
  VP := VP-2.0;
  B := STS*VP/XI*(1.0+B);
_15:
  inc(I2,2);
 end;
_16:
B := DP*X1*(1.0+B);
if ( N2 = 1 ) 
 then goto _18;
I2 := N2 div 2;
C := 1.0;
for I:=1 to I2 do
 begin
  B := B*X1*C/(C-0.5);
  C := C+1.0;
_17:
 end;
_18:
X3 := SYS015*(A-B);
Result := X3;
_19:
if ( X3 < 0.0 ) 
 then Result := 0.0;
goto _25;
_20:
IERR := 65;
goto _22;
_21:
IERR := 66;
_22:
Result := SYS059;
goto _24;
_23:
Result := 0.0;
goto _25;
_24:
UTSF12(IERR,42);
_25:
exit;
end;

end.
